import logging
import os

log = logging.getLogger('utils.zipx')


def download_zip(download_url, zip_file):
    os.system('wget %s -O %s ' % (download_url, zip_file))
    log.info(
        'Downloaded zip from %s to %s',
        download_url,
        zip_file,
    )


def unzip(zip_file, unzip_dir):
    os.system(
        'unzip -d %s -o %s'
        % (
            unzip_dir,
            zip_file,
        )
    )
    log.info('Unzipped %s to %s', zip_file, unzip_dir)



import math

from utils import geo

PRECISION = 6
ABS_TOL = 0.1 ** PRECISION


class Bounds:
    def __init__(self, min_latlng, max_latlng):
        self.min_latlng = min_latlng
        self.max_latlng = max_latlng

    @property
    def min_lat(self):
        return self.min_latlng.lat

    @property
    def min_lng(self):
        return self.min_latlng.lng

    @property
    def max_lat(self):
        return self.max_latlng.lat

    @property
    def max_lng(self):
        return self.max_latlng.lng

    def __add__(self, other):
        return Bounds(
            LatLng(
                min(self.min_lat, other.min_lat),
                min(self.min_lng, other.min_lng),
            ),
            LatLng(
                max(self.max_lat, other.max_lat),
                max(self.max_lng, other.max_lng),
            ),
        )

    @property
    def span(self):
        return [self.max_lat - self.min_lat, self.max_lng - self.min_lng]

    def __eq__(self, other):
        return all([
            self.min_latlng == other.min_latlng,
            self.max_latlng == other.max_latlng,
        ])

    def __str__(self):
        return str([self.min_latlng, self.max_latlng])


class LatLng:
    def __init__(self, lat, lng):
        self.lat = lat
        self.lng = lng

    def __str__(self):
        return f'({self.lat:.6f}, {self.lng:.6f})'

    def __eq__(self, other):
        return all([math.isclose(self.lat, other.lat, abs_tol=ABS_TOL),
                   math.isclose(self.lng, other.lng, abs_tol=ABS_TOL), ])

    @property
    def raw(self):
        return [self.lat, self.lng]

    @property
    def bounds(self):
        return Bounds(self, self)

    def distance(self, other):
        return geo.get_distance(self.raw, other.raw)


class LatLngIndex:
    POINT_PEDRO = LatLng(9.835556, 80.212222)
    DONDRA_HEAD = LatLng(5.923389, 80.589694)
    SANGAMAN_KANDA = LatLng(7.022222, 81.879167)
    KANCHCHATHEEVU = LatLng(9.383333, 79.516667)


class Polygon:
    def __init__(self, latlng_list):
        self.latlng_list = latlng_list

    def __iter__(self):
        for latlng in self.latlng_list:
            yield latlng

    def __len__(self):
        return len(self.latlng_list)

    @property
    def bounds(self):
        bounds = None
        for latlng in self:
            if not bounds:
                bounds = latlng.bounds
            else:
                bounds = bounds + latlng.bounds
        return bounds



"""Time utils."""
import datetime
import time

DEFAULT_TIME_FORMAT = '%Y-%m-%d %H:%M:%S'
FORMAT_DATE = '%Y-%m-%d'
FORMAT_DATE_ID = '%Y%m%d'

TIMEZONE_OFFSET_LK = -19_800
TIMEZONE_OFFSET_GMT = 0


class SECONDS_IN:
    """Units of time."""
    MINUTE = 60
    HOUR = 3600
    DAY = 86400
    WEEK = 7 * 86400
    FORTNIGHT = 14 * 86400
    MONTH = 365.25 * 4 * 86400 / 12
    QTR = 365.25 * 86400 / 4
    YEAR = 365.25 * 86400


class AVG_DAYS_IN:
    WEEK = 7
    MONTH = 365.25 / 12
    YEAR = 365.25


def get_timezone_offset(timezone=None):
    if timezone is None:
        return 0
    return timezone - time.timezone


def get_timezone():
    """Get timezone."""
    return str(datetime.datetime.now().astimezone().tzinfo)


def get_unixtime():
    """Get current unixtime."""
    return (int)(time.time())


def parse_time(
    time_str,
    time_format=DEFAULT_TIME_FORMAT,
    timezone=None,
):
    """Parse time string, and return unixtime."""
    _datetime = datetime.datetime.strptime(time_str, time_format)
    return (int)(time.mktime(_datetime.timetuple())) \
        + get_timezone_offset(timezone)


def format_time(
    unixtime,
    time_format=DEFAULT_TIME_FORMAT,
    timezone=None,
):
    """Format unixtime as time string."""
    _datetime = datetime.datetime.fromtimestamp(
        unixtime - get_timezone_offset(timezone),
    )
    return _datetime.strftime(time_format)


def format_current_date_with_timezone():
    """Format date with timezone."""
    date = format_time(get_unixtime(), '%B %d, %Y %H:%M%p')
    timezone = get_timezone()
    return '{date} {timezone}'.format(date=date, timezone=timezone)


def get_date_id(unixtime=None, timezone=None,):
    """Get date id."""
    if unixtime is None:
        unixtime = get_unixtime()
    return format_time(unixtime, FORMAT_DATE_ID, timezone)


def parse_date_id(date_id, timezone=None):
    return parse_time(date_id, FORMAT_DATE_ID, timezone)


def get_date(unixtime=None, timezone=None):
    """Get date id."""
    if unixtime is None:
        unixtime = get_unixtime()
    return format_time(unixtime, FORMAT_DATE, timezone)



"""Filex."""

import random

from utils import timex

MIN_INT, MAX_INT = 10 ** 15, 10 ** 16 - 1


def read(file_name):
    """Read."""
    with open(file_name, 'r') as fin:
        content = fin.read()
        fin.close()
        return content


def write(file_name, content, mode='w'):
    """Write."""
    with open(file_name, mode) as fout:
        fout.write(content)
        fout.close()


def get_tmp_file():
    """Get tmp file name."""
    return '/tmp/tmp.%s.%d' % (
        timex.format_time(timex.get_unixtime(), '%Y%m%d%H%M%S'),
        random.randint(MIN_INT, MAX_INT),
    )



"""Utils."""

import logging

from py_console import console, textColor


def get_level_color(levelno):
    return {
        logging.CRITICAL: textColor.MAGENTA,
        logging.ERROR: textColor.RED,
        logging.WARNING: textColor.YELLOW,
        logging.INFO: textColor.GREEN,
        logging.DEBUG: textColor.WHITE,
        logging.NOTSET: textColor.WHITE,
    }.get(levelno, textColor.WHITE)


class CustomLoggingFormatter(logging.Formatter):
    def format(self, record):
        color = get_level_color(record.levelno)
        return console.highlight(
            f'({record.name}-{record.levelname}): {record.msg}',
            color)


def get_logger(logger_name='custom'):
    formatter = CustomLoggingFormatter()
    sh = logging.StreamHandler()
    sh.setLevel(logging.DEBUG)
    sh.setFormatter(formatter)

    log = logging.getLogger(logger_name)
    log.propagate = False
    log.setLevel(logging.DEBUG)
    log.handlers = [sh]
    return log


if __name__ == '__main__':
    log = get_logger('testing')
    log.critical('This is CRITICAL')
    log.error('This is ERROR')
    log.warning('This is WARNING')
    log.info('This is INFO')
    log.debug('This is DEBUG')
    print('This is a regular print')



import xml.dom.minidom as minidom
import xml.etree.ElementTree as ElementTree

from utils import dt, filex

FONT_FAMILY = 'Gill Sans'
DEFAULT_ATTRIB_MAP = {
    'html': {
        'style': 'font-family: %s;' % FONT_FAMILY,
    },
    'svg': {
        'xmlns': 'http://www.w3.org/2000/svg',
    },
    'text': {
        'font-family': FONT_FAMILY,
        'text-anchor': 'middle',
        'fill': '#000000',
        'stroke': 'none',
    },
    'text-gray': {
        'font-family': FONT_FAMILY,
        'text-anchor': 'middle',
        'fill': '#c0c0c0',
        'stroke': 'none',
    },
}


def render_link_styles(css_file='styles.css'):
    return _('link', None, {'rel': 'stylesheet', 'href': css_file})


def style(**kwargs):
    style_content = ''.join(list(map(
        lambda item: '%s:%s;' % (dt.to_kebab(str(item[0])), str(item[1])),
        kwargs.items(),
    )))
    return dict(style=style_content)


class _:
    def __init__(
        self,
        tag,
        child_list_or_str_or_other=None,
        attrib_custom={},
    ):
        """XML Element."""
        tag_real = tag.split('-')[0]

        attrib = DEFAULT_ATTRIB_MAP.get(tag, {})
        attrib.update(attrib_custom)
        attrib = dict(zip(attrib.keys(), list(map(str, attrib.values()))))

        element = ElementTree.Element(tag_real)
        element.attrib = attrib

        if isinstance(child_list_or_str_or_other, list):
            child_list = child_list_or_str_or_other
            child_element_list = list(map(
                lambda child: child.element,
                list(filter(
                    lambda child_or_none: child_or_none is not None,
                    child_list,
                ))
            ))
            for child_element in child_element_list:
                element.append(child_element)

        elif isinstance(child_list_or_str_or_other, str):
            element.text = str(child_list_or_str_or_other)

        self.__element__ = element

    @property
    def element(self):
        return self.__element__

    def __str__(self):
        s = ElementTree.tostring(self.element, encoding='utf-8').decode()
        parsed_s = minidom.parseString(s)
        return parsed_s.toprettyxml(indent='  ')

    def __repr__(self):
        return self.__str__()

    def store(self, xml_file):
        filex.write(xml_file, str(self))



"""Utils related to lists and dicts."""


def dict_list_to_index(dict_list, key):
    """Given a list of dicts, returns an index mapping each dict item
    to the dict.
    """
    return dict(
        zip(
            list(
                map(
                    lambda d: d.get(key, None),
                    dict_list,
                )
            ),
            dict_list,
        )
    )


def unique(lst):
    """Get unique values from list."""
    return list(set(lst))


def flatten(list_of_list):
    """Flatten list of lists."""
    flattened_list = []
    for lst in list_of_list:
        flattened_list += lst
    return flattened_list


def sort_dict_items_by_key(_dict):
    """Sort dict items by key."""
    return sorted(
        _dict.items(),
        key=lambda item: item[0],
    )


def dict_get(_dict, keys):
    """Get dict values by keys."""
    return [_dict[key] for key in keys]


def dict_list_get_values_for_key(dict_list, key):
    """Get values for keys."""
    return [d[key] for d in dict_list]


def dict_set(_dict, keys, values):
    """Set dict values by keys."""
    for key, value in zip(keys, values):
        _dict[key] = value
    return _dict


def get_count(iter, func_get_keys):
    """Count the instances of some arbitrary attribute among an iter's items"""
    key_to_count = {}
    for item in iter:
        keys = func_get_keys(item)
        for key in keys:
            key_to_count[key] = key_to_count.get(key, 0) + 1
    return key_to_count



import colorsys
import random

PRECISION = 2


def random_hsl_vector(hue=None, saturation=None, lightness=None):
    hue0 = round(random.random(), PRECISION)
    saturation0 = 1
    lightness0 = 0.8

    hue = hue0 if (hue is None) else hue
    saturation = saturation0 if (saturation is None) else saturation
    lightness = lightness0 if (lightness is None) else lightness

    return (hue, saturation, lightness)


def random_rgb_vector(red=None, green=None, blue=None):
    (hue, saturation, lightness) = random_hsl_vector()
    (red0, green0, blue0) = list(map(
        lambda x: (int)(x * 255),
        colorsys.hls_to_rgb(hue, lightness, saturation),
    ))

    red = red0 if (red is None) else red
    green = green0 if (green is None) else green
    blue = blue0 if (blue is None) else blue

    return (red, green, blue)


def random_hsl(hue=None, saturation=None, lightness=None):
    (hue, saturation, lightness) = random_hsl_vector(hue, saturation, lightness)
    return f'hsl({hue:.0f},{saturation:.0%},{lightness:.0%})'


def random_rgb(red=None, green=None, blue=None):
    (red, green, blue) = random_rgb_vector(red, green, blue)
    return f'rgb({red},{green},{blue})'


def random_hex(red=None, green=None, blue=None):
    (r, g, b) = random_rgb_vector(red, green, blue)
    return f'#{r:02x}{g:02x}{b:02x}'



"""Image utils."""
import logging

from PIL import Image

log = logging.getLogger('imagex')
logging.basicConfig(level=logging.INFO)


def crop(
    original_image_file,
    cropped_image_file,
    left_top,
    width_height,
):
    """Crop."""
    left, top = left_top
    width, height = width_height
    _im = Image.open(original_image_file)
    cropped_im = _im.crop((left, top, left + width, top + height))
    cropped_im.save(cropped_image_file)
    log.info('Saved cropped image to %s', cropped_image_file)


def resize(
    original_image_file,
    resized_image_file,
    width_height,
):
    """Resize."""
    _im = Image.open(original_image_file)
    resized_im = _im.resize(width_height)
    resized_im.save(resized_image_file)
    log.info('Saved resized image to %s', resized_image_file)
